<?php

namespace app\traits\account;

use app\model\AccountTag;
use app\model\AccountTagPivot;
use Exception;
use think\Collection;
use think\db\exception\DataNotFoundException;
use think\db\exception\DbException;
use think\db\exception\ModelNotFoundException;
use think\Model;

/**
 * 标签相关
 *
 * Trait TagTrait
 * @package app\traits\account
 */
trait TagTrait
{
    /**
     * 批量设置标签
     *
     * @param  array  $accountIds 用户ID列表
     * @param  array  $tagIds 标签ID列表
     * @return bool
     * @throws Exception
     */
    public function setTagByBatch(array $accountIds, array $tagIds): bool
    {
        // 新增标签
        $insert = [];
        foreach ($accountIds as $id) {
            foreach ($tagIds as $tag) {
                $arr               = [];
                $arr['account_id'] = $id;
                $arr['tag_id']     = $tag;

                $insert[] = $arr;
            }
        }

        if (!empty($insert)) {
            // 删除所有人标签
            AccountTagPivot::whereIn('account_id', $accountIds)->delete();
            (new AccountTagPivot())->saveAll($insert);
        }

        return true;
    }

    /**
     * 获取标签列表
     *
     * @return AccountTag[]|array|Collection
     * @throws DataNotFoundException
     * @throws DbException
     * @throws ModelNotFoundException
     */
    public function getTagList()
    {
        return AccountTag::order('sort', 'desc')->order('id', 'asc')->select();
    }
}